package cmd

import (
	"github.com/rs/zerolog"
	"orus.io/orus-io/rednerd/engines/mjml"
	"orus.io/orus-io/rednerd/engines/mustache"
	"orus.io/orus-io/rednerd/engines/wpd"
	"orus.io/orus-io/rednerd/rendering"
)

// SetupEngines creates and register the rendering engines
func SetupEngines(log zerolog.Logger) error {
	wpdEngine, err := wpd.NewEngine(log.With().Str("engine", "wpd").Logger())
	if err != nil {
		return err
	}
	if err := rendering.RegisterEngine("wpd", wpdEngine); err != nil {
		return err
	}

	if err := rendering.RegisterTemplateEngine("mustache", &mustache.Engine{}); err != nil {
		return err
	}

	mjmlEngine, err := mjml.NewEngine()
	if err != nil {
		return err
	}
	if err := rendering.RegisterEngine("mjml", mjmlEngine); err != nil {
		return err
	}

	return nil
}
