package cmd

import (
	"fmt"
	"os"

	flags "github.com/orus-io/go-flags"
)

// Run ...
func Run() error {
	defer func() {
		for _, cb := range OnShutdownCB {
			cb()
		}
	}()

	parser := flags.NewNamedParser("rednerd", flags.HelpFlag|flags.PassDoubleDash)
	parser.ShortDescription = "Rednerd API"
	parser.LongDescription = "The Rednerd rendering server"

	options := NewOptions(parser)
	if _, err := parser.AddGroup("Global", "", options); err != nil {
		return err
	}

	newSentryOptions(options).AddToParser(parser)

	serveCmd := NewServerCmd(options)
	if err := serveCmd.AddToParser(parser); err != nil {
		return err
	}

	var migrateCmd = MigrateCmd{
		options: options,
	}
	if _, err := parser.AddCommand(
		"migrate", "Init/migrate the database schema", "", &migrateCmd,
	); err != nil {
		log.Fatal().Msg(err.Error())
	}
	var adminPasswordCmd = AdminPasswordCmd{
		options: options,
	}
	if _, err := parser.AddCommand(
		"admin-password", "Set the admin password", "", &adminPasswordCmd,
	); err != nil {
		log.Fatal().Msg(err.Error())
	}

	var configFile = NewConfigWrite(parser)

	if _, err := parser.AddCommand(
		"generate-config", "Generate a configuration file", "", configFile,
	); err != nil {
		log.Fatal().Msg(err.Error())
	}

	if _, err := parser.Parse(); err != nil {
		code := 1
		if fe, ok := err.(*flags.Error); ok {
			if fe.Type == flags.ErrHelp {
				code = 0
				fmt.Println(err)
			} else {
				log.Error().Msg(err.Error())
			}
		} else {
			log.Err(err).Msg("failure")
		}
		os.Exit(code)
	}

	return nil
}
