package mustache

import (
	"context"

	"github.com/cbroglie/mustache"
	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/rendering"
)

// Engine is a TemplateEngine implementation on top of github.com/cbroglie/mustache
type Engine struct {
}

// Info ...
func (e Engine) Info() rendering.TemplateEngineInfo {
	return rendering.TemplateEngineInfo{
		Type:        "mustache",
		Language:    "mustache",
		Description: "Render a [mustache](https://mustache.github.io/) template.",
	}
}

// Load loads a template
func (e Engine) Load(ctx context.Context, template *models.Template) (rendering.Template, error) {
	s, err := template.String()
	if err != nil {
		return nil, err
	}
	bodyTemplate, err := mustache.ParseString(s)
	if err != nil {
		return nil, err
	}
	t := Template{
		outputType:        template.Produces,
		metadata:          template.StaticMetadata,
		metadataTemplates: make(map[string]*mustache.Template),
		bodyTemplate:      bodyTemplate,
	}
	for name, meta := range template.Metadata {
		tmpl, err := mustache.ParseString(meta)
		if err != nil {
			return nil, err
		}
		t.metadataTemplates[name] = tmpl
	}
	return &t, nil
}

// Template ...
type Template struct {
	outputType        string
	metadata          models.Metadata
	metadataTemplates map[string]*mustache.Template
	bodyTemplate      *mustache.Template
}

// Render the template
func (t Template) Render(ctx context.Context, record models.Record) (*models.Document, error) {
	var output = models.NewDocument(t.outputType)

	output.UpdateMetadata(t.metadata)

	for key, tmpl := range t.metadataTemplates {
		s, err := tmpl.Render(record)
		if err != nil {
			return nil, err
		}
		output.SetMetadata(key, s)
	}

	body, err := t.bodyTemplate.Render(record)
	if err != nil {
		return nil, err
	}
	output.SetString(body)

	return output, nil
}
