package handlers

import (
	"context"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/rednerd/database"
	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/user_management"
)

// NewUserApikeyListHandler returns a UserApikeyListHandler
func NewUserApikeyListHandler(db *sqlx.DB) *UserApikeyListHandler {
	return &UserApikeyListHandler{
		db: db,
	}
}

// UserApikeyListHandler handles GET /user
type UserApikeyListHandler struct {
	db *sqlx.DB
}

func (h *UserApikeyListHandler) handle(
	ctx context.Context, username string,
) ([]*models.Apikey, error) {
	var (
		log  = zerolog.Ctx(ctx)
		list []*models.Apikey
	)
	if err := database.SelectContext(ctx, h.db, &list,
		squirrel.
			Select(models.ApikeyColumns...).
			From(models.ApikeyDBTable).
			Where(squirrel.Eq{models.ApikeyDBUsernameColumn: username}),
		log,
	); err != nil {
		return nil, err
	}
	return list, nil
}

// Handle handles the incoming request
func (h *UserApikeyListHandler) Handle(
	params user_management.UserApikeyListParams, principal *models.Principal,
) middleware.Responder {
	if !principal.IsAdmin {
		return user_management.NewUserApikeyListUnauthorized()
	}
	ctx := params.HTTPRequest.Context()
	log := zerolog.Ctx(ctx)
	list, err := h.handle(params.HTTPRequest.Context(), params.Username)
	if err != nil {
		return InternalError(err, log, false)
	}
	return user_management.NewUserApikeyListOK().WithPayload(list)
}
