package handlers

import (
	"context"

	"github.com/Masterminds/squirrel"
	"github.com/go-openapi/runtime/middleware"
	"github.com/jmoiron/sqlx"
	"github.com/rs/zerolog"
	"orus.io/orus-io/rednerd/database"
	"orus.io/orus-io/rednerd/models"
	"orus.io/orus-io/rednerd/restapi/operations/user_management"
)

// NewUserGetHandler returns a UserGetHandler
func NewUserGetHandler(db *sqlx.DB) *UserGetHandler {
	return &UserGetHandler{
		db: db,
	}
}

// UserGetHandler handles GET /user
type UserGetHandler struct {
	db *sqlx.DB
}

func (h *UserGetHandler) handle(
	ctx context.Context, username string,
) (*models.Userinfo, error) {
	var (
		log  = zerolog.Ctx(ctx)
		user models.Userinfo
	)
	if err := database.GetContext(ctx, h.db, &user,
		squirrel.Select(models.UserinfoColumns...).
			From(models.UserDBTable).
			Where(squirrel.Eq{models.UserinfoUsernameColumn: username}),
		log,
	); err != nil {
		return nil, err
	}
	return &user, nil
}

// Handle handles the incoming request
func (h *UserGetHandler) Handle(
	params user_management.UserGetParams, principal *models.Principal,
) middleware.Responder {
	if !principal.IsAdmin {
		return user_management.NewUserGetUnauthorized()
	}
	ctx := params.HTTPRequest.Context()
	log := zerolog.Ctx(ctx)
	user, err := h.handle(params.HTTPRequest.Context(), params.Username)
	if err != nil {
		return InternalError(err, log, false)
	}
	return user_management.NewUserGetOK().WithPayload(user)
}
