package tests

import (
	"net/http"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	"orus.io/orus-io/rednerd/models"
)

func TestTemplateManagement(t *testing.T) {
	tester := NewAPITester(t)
	defer tester.Close()

	defer tester.Logout()
	tester.Login("admin", "admin")

	tester.CreateUser("janedoe", "janedoe")
	tester.Login("janedoe", "janedoe")

	_ = tester.Run("Create",
		func(t *testing.T) {
			var template models.Template
			require.Equal(t, http.StatusCreated,
				tester.MustPostJSON("/api/v1/template/janedoe", JSONObj{
					"name":        "t1",
					"language":    "mustache",
					"produces":    "text/html",
					"body-format": "text",
					"body":        "<html><body>Hi {{name}}!</body></html>",
				}, &template))
			assert.Equal(t, "t1", template.Name)
			assert.Equal(t, "janedoe", template.Account)
		}) &&
		tester.Run("Get",
			func(t *testing.T) {
				var template models.Template
				require.Equal(t, http.StatusOK,
					tester.MustGetJSON("/api/v1/template/janedoe/t1", &template),
				)
				assert.Equal(t, "t1", template.Name)
				assert.Equal(t, "janedoe", template.Account)
			}) &&
		tester.Run("List",
			func(t *testing.T) {
				var templateList []models.Template
				require.Equal(t, http.StatusOK,
					tester.MustGetJSON("/api/v1/template/janedoe", &templateList))
				assert.Equal(t, 1, len(templateList))
				assert.Equal(t, "t1", templateList[0].Name)
			}) &&
		tester.Run("Update",
			func(t *testing.T) {
				var template models.Template
				require.Equal(t, http.StatusOK,
					tester.MustPutJSON("/api/v1/template/janedoe/t1", JSONObj{
						"language":    "mustache",
						"produces":    "text/mjml",
						"body-format": "text",
						"body":        "<mjml><mj-text>hello {{name}}</mj-text></mjml>",
					}, &template))
				assert.Equal(t, "t1", template.Name)
				assert.Equal(t, "janedoe", template.Account)
				assert.Equal(t, "text/mjml", template.Produces)
			}) &&
		tester.Run("Rename",
			func(t *testing.T) {
				var template models.Template
				require.Equal(t, http.StatusOK,
					tester.MustPutJSON("/api/v1/template/janedoe/t1", JSONObj{
						"name":        "t2",
						"language":    "mustache",
						"produces":    "text/html",
						"body-format": "text",
						"body":        "<html><body>Hi {{name}}!</body></html>",
					}, &template))
				assert.Equal(t, "t2", template.Name)
				assert.Equal(t, "janedoe", template.Account)

				require.Equal(t, http.StatusOK,
					tester.MustGetJSON("/api/v1/template/janedoe/t2", &template),
				)
				assert.Equal(t, "t2", template.Name)
				assert.Equal(t, "janedoe", template.Account)
			})
}
